﻿using GrapeCity.ActiveReports;
using System;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Xml;

namespace ReportResourceLocator
{
    internal sealed class CustomResourceLocator : ResourceLocator
    {
        public override Resource GetResource(ResourceInfo resourceInfo)
        {
            Resource resource;
            string name = resourceInfo.Name;
            Uri uri = new Uri(name);
            Debug.WriteLine(uri);
            if (uri.GetLeftPart(UriPartial.Scheme).StartsWith("Report:", true, CultureInfo.InvariantCulture))
            {
                Stream stream = GetReportFromSpecialFolder(uri);
                if (stream == null)
                {
                    stream = new MemoryStream();
                }
                resource = new Resource(stream, uri);
            }
            return resource;
        }

        private static Stream GetReportFromSpecialFolder(Uri path)
        {
            int startPathPos = "Report:".Length;
            if (startPathPos >= path.ToString().Length)
            {
                return null;
            }
            string reportName = path.ToString().Substring(startPathPos);
            string myReportsPath = Application.StartupPath;
            if (!myReportsPath.EndsWith("\\")) myReportsPath += "\\";
            string reportsPath = Path.Combine(myReportsPath, reportName);
            if (!File.Exists(reportsPath)) return null;
            MemoryStream stream;
            try
            {
                PageReport report = new PageReport(new FileInfo(reportsPath));
                stream = new MemoryStream();
                var writer = XmlWriter.Create(stream);
                report.Save(writer);
                writer.Flush();
                stream.Position = 0;
            }
            catch (OutOfMemoryException)
            {
                return null;
            }
            catch (ExternalException)
            {
                return null;
            }
            return stream;
        }
    }
}
